var WordCountPlusKeys = {
	
	accel: "",
	shift: "",
	control: "",
	alt: "",
	meta: "",
	separator: "",

	init: function() {
		// Get the OS-specific string representations of the modifier keys and the modifier separator
		var platformKeys = document.getElementById("platformKeys");
		this.shift = platformKeys.getString("VK_SHIFT");
		this.control = platformKeys.getString("VK_CONTROL");
		this.alt = platformKeys.getString("VK_ALT");
		this.meta = platformKeys.getString("VK_META");
		this.separator = platformKeys.getString("MODIFIER_SEPARATOR");

		const accelKey = Components.classes["@mozilla.org/preferences-service;1"].
		                 getService(Components.interfaces.nsIPrefService).
		                 getIntPref("ui.key.accelKey");

		// The accelorator key is different depending on OS
		switch (accelKey) {
			case 17:
				this.accel = this.control;
				break;
			case 18:
				this.accel = this.alt;
				break;
			case 224:
				this.accel = this.meta;
				break;
			default:
				this.accel = "Accel";
		}

		// the hidden modifiers and key boxes get their values automatically from preferences,
		// but the shortcut box needs to take those values and put them together for display
		this.changeShortcutBox("count");
		this.changeShortcutBox("add");
		this.changeShortcutBox("reset");
		
		// SJW:20080718
		// For some reason, a description box that is set through a locale string and ends up wrapping to multiple
		// lines does not cause the groupbox or window to recognize the additional vertical space it takes up.
		// sizeToContent has no effect unless the computed css height is grabbed and forecefully assigned.
		var shortcutsgroup = document.getElementById('wordcountplus-shortcutsgroup');
		var groupHeight = parseFloat(document.defaultView.getComputedStyle(shortcutsgroup, null).getPropertyValue('height'));
		shortcutsgroup.setAttribute("height", groupHeight);
		window.sizeToContent();
	},

	handleKeyPress: function(name, event)	{
		if (event.charCode && (event.ctrlKey || event.altKey || event.metaKey)) {
			var modifiers = "";
			if (event.ctrlKey) modifiers += "control,";
			if (event.metaKey) modifiers += "meta,";
			if (event.shiftKey) modifiers += "shift,";
			if (event.altKey) modifiers += "alt,";
			// strip off final comma
			modifiers = modifiers.replace(/,$/, "");

			var modifiersCtrl = document.getElementById("wordcountplus-modifiers-" + name);
			modifiersCtrl.value = modifiers;
			modifiersCtrl.doCommand();

			var keyCtrl = document.getElementById("wordcountplus-key-" + name);
			keyCtrl.value = String.fromCharCode(event.charCode).toUpperCase();
			keyCtrl.doCommand();

			event.preventDefault();
			this.changeShortcutBox(name);
		}
	},

	changeShortcutBox: function(name) {
		var shortcut = document.getElementById("wordcountplus-modifiers-" + name).value;
		
		// replace comma with separator string
		shortcut = shortcut.replace(/[\s,]+/g, this.separator);
		// translate FF modifier names into OS-specific strings
		shortcut = shortcut.replace(/accel/, this.accel);
		shortcut = shortcut.replace(/shift/, this.shift);
		shortcut = shortcut.replace(/control/, this.control);
		shortcut = shortcut.replace(/alt/, this.alt);
		shortcut = shortcut.replace(/meta/, this.meta); 
		
		// The shortcut display is the translated modifiers plus the separator plus the key value
		shortcut += this.separator + document.getElementById("wordcountplus-key-" + name).value;
		document.getElementById("wordcountplus-shortcut-" + name).value = shortcut;
	},
	
	setKeyDefault: function(name) {
		// SJW:20080716
		// Probably there is a better way to do this? But I cannot see how to get the default pref
		// without first locking it. So lock it, get the default, unlock it, check to see if we need to
		// swap Alt for Ctrl (Macs only) then set the default on the pref boxes and the display box.
		var opts = Components.classes["@mozilla.org/preferences-service;1"].
         				getService(Components.interfaces.nsIPrefService).
		         		getBranch("extensions.wordcountplus.options.");
		
		// Can prefs be locked by users non-programmatically? Whatever, let's not assume anything
		// and preserve the locked/unlocked state.
		var locked = opts.prefIsLocked(name + "Modifiers");
		opts.lockPref(name + "Modifiers");
		var defaultModifiers = opts.getCharPref(name + "Modifiers");
		if (locked == false) {
			opts.unlockPref(name + "Modifiers");		
		}
		
		locked = opts.prefIsLocked(name + "Key");
		opts.lockPref(name + "Key");
		var defaultKey = opts.getCharPref(name + "Key");
		if (locked == false) {
			opts.unlockPref(name + "Key");
		}
		
		if (MacDefaults.isMac()) {	
			defaultModifiers = MacDefaults.altToCtrl(defaultModifiers);
		}
		document.getElementById("wordcountplus-modifiers-" + name).value = defaultModifiers;
		document.getElementById("wordcountplus-modifiers-" + name).doCommand();
		document.getElementById("wordcountplus-key-" + name).value = defaultKey;	
		document.getElementById("wordcountplus-key-" + name).doCommand();	
		this.changeShortcutBox(name);
	}
	
};