var WordCountPlus = {

	selectedWords: 0,
	previousWords: -1,
	totalWords: 0,
	previousTotal: -1,
	selectedChars: 0,
	previousChars: -1,
	totalChars: 0,
	previousCharTotal: -1,
	manualReset: false,
	previousManualReset: false,
	undoAllowed: false,
	autoCopy: false,
	autoDeselect: false,
	preCountText: "",
	postCountText: "",
	thousandSep: "",
	prefs: null,
	options: null,
	observerService: null,
	clipboardHelper: null,

	init: function() {	
		// Get just the wordcountplus prefs, so they're simpler to get and set
		this.prefs = Components.classes["@mozilla.org/preferences-service;1"]
         						.getService(Components.interfaces.nsIPrefService)
		         				.getBranch("extensions.wordcountplus.");		         				
		this.getCounterState();

		// Get and monitor the preferences that are set in the options dialogue
		this.options = Components.classes["@mozilla.org/preferences-service;1"]
         						.getService(Components.interfaces.nsIPrefService)
		         				.getBranch("extensions.wordcountplus.options.");
		this.options.QueryInterface(Components.interfaces.nsIPrefBranch2);
		this.options.addObserver("", this, false);
		this.getOptions();
		this.updateKeys();
		
		// Show Word Count Plus in the status bar
		this.updateStatus();
		
		// Watch for when the main area context menu is showing -- need to hide menu options if no text selected
		// SJ:20080709
		// Give the event listener a function with the correct context for "this".
		document.getElementById("contentAreaContextMenu").addEventListener("popupshowing", function() { WordCountPlus.contextItemsHide(); }, false);
		
		// Watch for the application quit notification
		this.observerService = Components.classes["@mozilla.org/observer-service;1"]
	                                  .getService(Components.interfaces.nsIObserverService);
		this.observerService.addObserver(this, "quit-application-granted", false);
		
		// Set up access to the clipboard
		this.clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
	},
	
	// set up in the commandset to be called anytime the select event fires in the window
	updateCommands: function() {
		var noSelection = (this.getSelectedText() === null) ? true : false;
		document.getElementById("cmd_wordcountplusCount").setAttribute("disabled", noSelection);
		document.getElementById("cmd_wordcountplusAddCount").setAttribute("disabled", noSelection);	
	},
	
	handleStatusClick: function(clickEvent) {
		// SJW:20080730
		// These could  be triggered by invoking the commands, but it's not immediately obvious
		// that clicking the icon should do anything unless it reacts in some way ANY time you click it, 
		// including the case when there is no selection. So instead of disabling the click actions when
		// there is no selection, like we do with the keyboard commands and the menu items, let them go 
		// through and toss up the no selection alert.
		if (clickEvent.button == 0 && clickEvent.shiftKey) {
			// on shift-left-click, count total words
			this.countTotalWords();
		}
		else if ((clickEvent.button == 1) || (clickEvent.button == 0 &&  clickEvent.altKey)) {
			// on middle click or alt-left-click, reset the counts 
			// SJW:20080730
			// This one can use the command, though, because there's no visual repercussion when there's nothing to reset,
			// so may as well keep in tune with the disabling/enabling of the command.
			var resetMenuItem = document.getElementById("cmd_wordcountplusReset");
			if (resetMenuItem.getAttribute("disabled") == "false") {
				resetMenuItem.doCommand();
			}
		}
		else if (clickEvent.button == 0) {
			// on left click, no key modifiers, count selected words
			this.countSelectedWords();
		}
	},
	
	openOptionsWindow: function() {
		// Only open the options window if it's not already open -- otherwise just focus it              
		var wm = Components.classes['@mozilla.org/appshell/window-mediator;1']
                        .getService(Components.interfaces.nsIWindowMediator);
		var optionsWin = wm.getMostRecentWindow("mozilla:wordcountplus-options");
 		if (optionsWin) {
 			optionsWin.focus();
		}
		else {
			var instantApply = Components.classes["@mozilla.org/preferences-service;1"].
		                 			getService(Components.interfaces.nsIPrefService).
		                 			getBoolPref("browser.preferences.instantApply", false);
			var features = "chrome,titlebar,resizable=no,centerscreen" + (instantApply ? ",dialog=no" : "");
			window.openDialog("chrome://wordcountplus/content/options.xul", "", features);
		}
	},
		
	showPanel: function() {
		// When a window is activated, retrieve the current counter state and 
		// make the wordcount panel visible, hiding it in inactive panes
		var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
             	.getService(Components.interfaces.nsIWindowMediator);
		var enumerator = wm.getEnumerator("navigator:browser");
		while(enumerator.hasMoreElements()) {
			var win = enumerator.getNext();
			win.document.getElementById("wordcountplus-panel").collapsed = true;
		}
		this.getCounterState();
		this.updateStatus();
		document.getElementById("wordcountplus-panel").collapsed = false;		
	},
	
	doCount: function(sel) {
		var c = sel;
		if (c == "") {
			c = 0;
		}
		else if (c != null) {
			// replace all whitespace with basic space chars, consolidate consecutive spaces to a single space, trim space off ends.
			c = c.replace(/\s/g,' ').replace(/ {2,}/g,' ').replace(/ $|^ /g,'');
			// SJW:20080730
			// Splitting an empty string returns an array with one undefined element, so return zero instead of splitting,
			// else it will return a false oount of one.
			c = (c == "") ? 0 : c.split(' ').length;
		}
		else {
			alert(document.getElementById("wordcountplus-strings").getString("nothingSelectedMessage"))
			c = -1;
		}
		return c;
	},
	
	getSelectedText: function() {						
		// SJW:20080728
		// Sometimes there could be no focused window.
		var w = document.commandDispatcher.focusedWindow;
		if (w) {
			// SJW:20080729
			// getSelection always returns *something* so test if it's collapsed to see if there's really a selection
			if (w.getSelection().isCollapsed == false) {
				return w.getSelection().toString();
			}			
		}
			
		// Nothing selected in the window, try for a text box
		var e = document.commandDispatcher.focusedElement;
		if (e) {
			if ((e.tagName.toLowerCase() == "input" && (e.type == "text")) || (e.tagName.toLowerCase() == "textarea") || (e.tagName.toLowerCase() == "textbox")) {
				var st = e.selectionStart;
				var end = e.selectionEnd;
				if (end > st) {
					return e.value.substring(st, end);
				}
			}
		}
		
		// SJW:20080729
		// There was no selection at all so return null, not empty string.
		return null;	
	},
	
	collapseSelected: function() {
		var c = document.commandDispatcher.focusedWindow.getSelection().toString();
		if (c) {
			document.commandDispatcher.focusedWindow.getSelection().collapseToStart();
		}
		else {
			// Nothing selected in the window, try for a text box
			var e = document.commandDispatcher.focusedElement;
			if (e) {
				if ((e.tagName.toLowerCase() == "input" && (e.type == "text")) || (e.tagName.toLowerCase() == "textarea") || (e.tagName.toLowerCase() == "textbox")) {
					e.setSelectionRange(e.selectionStart, e.selectionStart);
				}
			}
		}	
	},
	
	countSelectedWords: function() {	
		var sel = this.getSelectedText();
		var c = this.doCount(sel);
		if (c != -1) {
			this.previousWords = this.selectedWords;
			this.previousChars = this.selectedChars;	
			this.previousManualReset = this.manualReset;
			this.selectedWords = c;
			this.selectedChars = sel.length;
			this.manualReset = false;
			this.undoAllowed = true;
			
			// SJW:20080717
			// This is a simple count, so total count does not change. For undo purposes, the current
			// total is thus also the previous total.
			this.previousTotal = this.totalWords;
			this.reviousCharTotal = this.totalChars;
			
			this.prefs.setIntPref("selectedWords", this.selectedWords);
			this.prefs.setIntPref("previousWords", this.previousWords);
			this.prefs.setIntPref("totalWords", this.totalWords);
			this.prefs.setIntPref("previousTotal", this.previousTotal);
			this.prefs.setIntPref("selectedChars", this.selectedChars);
			this.prefs.setIntPref("previousChars", this.previousChars);
			this.prefs.setIntPref("totalChars", this.totalChars);
			this.prefs.setIntPref("previousCharTotal", this.previousCharTotal);
			this.prefs.setBoolPref("manualReset", this.manualReset);
			this.prefs.setBoolPref("previousManualReset", this.previousManualReset);
			this.prefs.setBoolPref("undoAllowed", this.undoAllowed);

			this.updateStatus();
			
			if (this.autoCopy) {
				this.doCopy(this.selectedWords);
			}
			
			if (this.autoDeselect) {
				// deselect the selection		
				this.collapseSelected();	
			}
		}
	},
	
	countTotalWords: function() {
		var existingWordCount = 0;
		var existingCharCount = 0;
		if (this.totalWords == 0) {
			existingWordCount = this.selectedWords;		
			existingCharCount = this.selectedChars;		
		}	
		var sel = this.getSelectedText();
		var c = this.doCount(sel);
		if (c != -1) {
			this.previousWords = this.selectedWords;
			this.selectedWords = c;
			this.previousTotal = this.totalWords;
			this.totalWords = this.totalWords + c + existingWordCount;
			this.previousChars = this.selectedChars;
			this.selectedChars = sel.length;
			this.previousCharTotal = this.totalChars;
			this.totalChars = this.totalChars + sel.length + existingCharCount;			
			this.previousManualReset = this.manualReset;
			this.manualReset = false;
			this.undoAllowed = true;
			this.prefs.setIntPref("selectedWords", this.selectedWords);
			this.prefs.setIntPref("previousWords", this.previousWords);
			this.prefs.setIntPref("totalWords", this.totalWords);
			this.prefs.setIntPref("previousTotal", this.previousTotal);
			this.prefs.setIntPref("selectedChars", this.selectedChars);
			this.prefs.setIntPref("previousChars", this.previousChars);
			this.prefs.setIntPref("totalChars", this.totalChars);
			this.prefs.setIntPref("previousCharTotal", this.previousCharTotal);
			this.prefs.setBoolPref("manualReset", this.manualReset);
			this.prefs.setBoolPref("previousManualReset", this.previousManualReset);
			this.prefs.setBoolPref("undoAllowed", this.undoAllowed);
			this.updateStatus();
			
			if (this.autoCopy) {
				this.doCopy(this.totalWords);
			}
			
			if (this.autoDeselect) {
				// deselect the selection	
				this.collapseSelected();			
			}
		}
	},
	
	zeroCounts: function() {
		// SJW:20080730
		// Even when both counts are already zero, we may need to change the status disply, since
		// a manual set to zero should clear the text completely, even if it's displaying 0 words 
		// (also, might be a char count even when 0 words). Switch the test for count != 0 or 
		// total !=0 to manualReset test. If manualReset is set, that means this call was just performed,
		// so don't do it again (shouldn't be possible with command disabled, but check anyway).
		if (!this.manualReset) {
			this.previousWords = this.selectedWords;
			this.previousChars = this.selectedChars;
			this.previousTotal = this.totalWords;
			this.previousCharTotal = this.totalChars;
			this.previousManualReset = this.manualReset;
			this.selectedWords = 0;
			this.selectedChars = 0;
			this.totalWords = 0;
			this.totalChars = 0;
			this.manualReset = true;
			this.undoAllowed = true;
			this.prefs.setIntPref("selectedWords", this.selectedWords);
			this.prefs.setIntPref("previousWords", this.previousWords);
			this.prefs.setIntPref("selectedChars", this.selectedChars);
			this.prefs.setIntPref("previousChars", this.previousChars);
			this.prefs.setIntPref("totalWords", this.totalWords);
			this.prefs.setIntPref("previousTotal", this.previousTotal);
			this.prefs.setIntPref("totalChars", this.totalChars);
			this.prefs.setIntPref("previousCharTotal", this.previousCharTotal);
			this.prefs.setBoolPref("manualReset", this.manualReset);
			this.prefs.setBoolPref("previousManualReset", this.previousManualReset);
			this.prefs.setBoolPref("undoAllowed", this.undoAllowed);
			this.updateStatus();
		}		
	},
	
	undoredoCount: function() {
		// SJW:20080730
		// Took out test for != zero -- need to be able to revert to zero now that counting is 
		// enabled when no text selected.
		var currentWords = this.selectedWords;
		var currentChars = this.selectedChars;
		var currentTotal = this.totalWords;
		var currentCharTotal = this.totalChars;
		var currentManualReset = this.manualReset;
		this.selectedWords = this.previousWords;
		this.previousWords = currentWords;
		this.selectedChars = this.previousChars;
		this.previousChars = currentChars;
		this.totalWords = this.previousTotal;
		this.previousTotal = currentTotal;
		this.totalChars = this.previousCharTotal;
		this.previousCharTotal = currentCharTotal;
		this.manualReset = this.previousManualReset;
		this.previousManualReset = currentManualReset;
		this.undoAllowed = !this.undoAllowed;
		this.prefs.setIntPref("selectedWords", this.selectedWords);
		this.prefs.setIntPref("previousWords", this.previousWords);
		this.prefs.setIntPref("selectedChars", this.selectedChars);
		this.prefs.setIntPref("previousChars", this.previousChars);
		this.prefs.setIntPref("totalWords", this.totalWords);
		this.prefs.setIntPref("previousTotal", this.previousTotal);
		this.prefs.setIntPref("totalChars", this.totalChars);
		this.prefs.setIntPref("previousCharTotal", this.previousCharTotal);
		this.prefs.setBoolPref("manualReset", this.manualReset);
		this.prefs.setBoolPref("previousManualReset", this.previousManualReset);
		this.prefs.setBoolPref("undoAllowed", this.undoAllowed);
		this.updateStatus();
	},
	
	// SJW:20080730
	// When a reset to zero has been triggered, clear all text and just show the icon. Any other time a
	// count or total is zero, though, must show that as text. Use resetFlag to differentiate.
	updateStatus: function() {
		var wordStatus = document.getElementById("wordcountplus-panel");
		var wordtext = "";
		if ((this.selectedWords != 0) || (this.selectedWords == 0 && !this.manualReset)) {
			wordtext = "words";
			switch (this.selectedWords) {
				case 0:
				 wordtext = document.getElementById("wordcountplus-strings").getString("wordsZero");
				 break;
				case 1:
				 wordtext = document.getElementById("wordcountplus-strings").getString("wordsOne");
				 break;
				case 2:
				 wordtext = document.getElementById("wordcountplus-strings").getString("wordsTwo");
				 break;
				default:
				 wordtext = document.getElementById("wordcountplus-strings").getString("wordsMore");
			}
			wordtext = this.addThousandSep(this.selectedWords) + ' ' + wordtext;
		}
		if (this.totalWords != 0) {			
			var totaltext = document.getElementById("wordcountplus-strings").getString("totalWords");
			wordtext = (wordtext != "") ? wordtext + ' | ' : wordtext;
			wordtext = wordtext + this.addThousandSep(this.totalWords) + ' ' + totaltext;
		}
		
		// Need to change the style of the panel depending on whether there is text or not, to 
		// get nice even spacing in all cases.
		// SJW:20080715
		// Also take this opportunity to disable/enable the reset command.
		var resetMenuItem = document.getElementById("cmd_wordcountplusReset");
		if (wordtext == "") {
			wordStatus.setAttribute("class", "statusbarpanel-iconic");
			wordStatus.setAttribute("src", "chrome://wordcountplus/skin/wcp-small.png");	
			resetMenuItem.setAttribute("disabled", "true");
		}
		else {
			wordStatus.setAttribute("class", "statusbarpanel-iconic-text");
			wordStatus.setAttribute("src", "chrome://wordcountplus/skin/wcp-small.png");
			resetMenuItem.setAttribute("disabled", "false");
		}
		wordStatus.label = wordtext;
	},
	
	updateTooltip: function() {
		var chartext = "";
		if (this.selectedChars != 0) {
			chartext = "chars";
			switch (this.selectedChars) {
				case 1:
				 chartext = document.getElementById("wordcountplus-strings").getString("charsOne");
				 break;
				case 2:
				 chartext = document.getElementById("wordcountplus-strings").getString("charsTwo");
				 break;
				default:
				 chartext = document.getElementById("wordcountplus-strings").getString("charsMore");
			}
			chartext = this.addThousandSep(this.selectedChars) + ' ' + chartext;
		}
		if (this.totalChars != 0) {			
			var totaltext = document.getElementById("wordcountplus-strings").getString("totalChars");
			chartext = (chartext != "") ? chartext + ' | ' : chartext;
			chartext = chartext + this.addThousandSep(this.totalChars) + ' ' + totaltext;
		}
		if (chartext == "") {
			chartext = document.getElementById("wordcountplus-strings").getString("wcpName");			
		}
		document.getElementById("wordcountplus-tooltiptext").setAttribute("value", chartext);
	},
	
	updateStatusContext: function() {		
		// SJW:20080715
		// Count actions are disabled/enabled via commands anytime the select event fires in the window
		// Reset is disabled/enabled via commands anytime the count or total is changed in the status panel display
		
		// Undo-Redo
		var undoMenuItem = document.getElementById("wordcountplus-undo");
		var redoMenuItem = document.getElementById("wordcountplus-redo");
		if (this.previousTotal < 0 && this.previousWords < 0) {
			undoMenuItem.setAttribute("disabled", "true");
			redoMenuItem.setAttribute("disabled", "true");
		}
		else {
			if (this.undoAllowed) {
				undoMenuItem.setAttribute("disabled", "false");
				redoMenuItem.setAttribute("disabled", "true");
			}
			else {
				undoMenuItem.setAttribute("disabled", "true");
				redoMenuItem.setAttribute("disabled", "false");
			}
		}		
		
		// Copy Words
		var copyselectedMenuItem = document.getElementById("wordcountplus-copyselected");
		if (this.selectedWords == 0) {
			copyselectedMenuItem.setAttribute("disabled", "true");
		}
		else {		
			copyselectedMenuItem.setAttribute("disabled", "false");
		}	
		
		// Copy Total
		var copytotalMenuItem = document.getElementById("wordcountplus-copytotal");
		if (this.totalWords == 0) {
			copytotalMenuItem.setAttribute("disabled", "true");
		}
		else {		
			copytotalMenuItem.setAttribute("disabled", "false");
		}	
		
		// Auto Copy
		var autocopyMenuItem = document.getElementById("wordcountplus-autocopy");
		autocopyMenuItem.setAttribute("checked", this.autoCopy);
		
		// Auto Deselect
		var autodeselectMenuItem = document.getElementById("wordcountplus-autodeselect");
		autodeselectMenuItem.setAttribute("checked", this.autoDeselect);
	},
		
	// These are hooked up to commands which get disabled/enabled automatically anytime the 
	// select event fires, but they should still be hidden when disabled to keep clutter down
	contextItemsHide: function() {
		var cmd = document.getElementById("cmd_wordcountplusCount");
		var noSelection = (cmd.getAttribute("disabled") == "true") ? true : false;
		document.getElementById("wordcountplus-count").hidden = noSelection;
		
		cmd = document.getElementById("cmd_wordcountplusAddCount");
		noSelection = (cmd.getAttribute("disabled") == "true") ? true : false;
		document.getElementById("wordcountplus-add").hidden = noSelection;
	},
		
	toggleAutoCopy: function(e) {
		e.preventDefault();
		e.stopPropagation();
		var ac = this.autoCopy; 
		document.getElementById("wordcountplus-autocopy").checked = !ac;
		this.autoCopy = !ac;
		this.prefs.setBoolPref("autoCopy", !ac);		
	},
	
	toggleAutoDeselect: function(e) {
		e.preventDefault();
		e.stopPropagation();
		var ads = this.autoDeselect;
		document.getElementById("wordcountplus-autodeselect").checked = !ads;
		this.autoDeselect = !ads;
		this.prefs.setBoolPref("autoDeselect", !ads);		
	},
	
	getCounterState: function() {	
		this.selectedWords = 0;		
		this.previousWords = -1;			
		this.totalWords = 0;
		this.previousTotal = -1;
		this.selectedChars = 0;	
		this.previousChars = -1;		
		this.totalChars = 0;
		this.previousCharTotal = -1;
		this.undoAllowed = false;
		this.autoCopy = false;
		this.autoDeselect = false;
		this.manualReset = true;
		this.previousManualReset = true;
		
		try {	
			this.selectedWords = this.prefs.getIntPref("selectedWords");
			this.previousWords = this.prefs.getIntPref("previousWords");
			this.totalWords = this.prefs.getIntPref("totalWords");
			this.previousTotal = this.prefs.getIntPref("previousTotal");
			this.selectedChars = this.prefs.getIntPref("selectedChars");
			this.previousChars = this.prefs.getIntPref("previousChars");
			this.totalChars = this.prefs.getIntPref("totalChars");
			this.previousCharTotal = this.prefs.getIntPref("previousCharTotal");
			this.manualReset = this.prefs.getBoolPref("manualReset");
			this.previousManualReset = this.prefs.getBoolPref("previousManualReset");
			this.undoAllowed = this.prefs.getBoolPref("undoAllowed");			
			this.autoCopy = this.prefs.getBoolPref("autoCopy");			
			this.autoDeselect = this.prefs.getBoolPref("autoDeselect");
		}
		catch(e) {}
	},
		
	getOptions: function() {
		this.preCountText = "";
		this.postCountText = "";
		this.thousandSep = "";
    	
		try {
			this.preCountText = this.options.getCharPref("textBeforeCount");
			this.postCountText = this.options.getCharPref("textAfterCount");
			this.thousandSep = this.options.getCharPref("thousandSep");
		}
		catch(e) {}
	},
	
	updateKeys: function() {
		// Grab the keyboard shortcuts from preferences. Defaults use Alt
		// but Mac OS needs to use Ctrl since Alt = Option = special chars.
		if (MacDefaults.isMac()) {
			MacDefaults.altToCtrlPref("countModifiers");
			MacDefaults.altToCtrlPref("addModifiers");
			MacDefaults.altToCtrlPref("resetModifiers");
		}
		
		var key;
		key = document.getElementById("key_wordcountplusCount");
		key.setAttribute("modifiers", this.options.getCharPref("countModifiers"));
		key.setAttribute("key", this.options.getCharPref("countKey"));
		
		key = document.getElementById("key_wordcountplusAddCount");
		key.setAttribute("modifiers", this.options.getCharPref("addModifiers"));
		key.setAttribute("key", this.options.getCharPref("addKey"));

		key = document.getElementById("key_wordcountplusReset");
		key.setAttribute("modifiers", this.options.getCharPref("resetModifiers"));
		key.setAttribute("key", this.options.getCharPref("resetKey"));	
	},
	
	doCopy: function(numWords) {
		var numString = this.customFormat(numWords);
		this.clipboardHelper.copyString(numString);
	},
	
	customFormat: function(numWords) {
		var numString = numWords;
		if (numString != "") {
			numString = this.addThousandSep(numWords);
			numString = this.preCountText + numString + this.postCountText;
		}
		return numString;
	},
	
	addThousandSep: function(numWords) {
		var numString = numWords.toString();
		if ((numString != "") && (this.thousandSep != "")) {
			numString = numString.replace(/(\d)(?=(\d{3})+([.]|$))/g,"$1"+this.thousandSep);
		}
		return numString;
	},
	
	observe: function(subject, topic, data) {
		// Respond to pertinent notifications
		switch(topic) {
			case "nsPref:changed":
			// The preferences were changed in the Options dialogue
			// Can't make keyboard shortcuts affect current window so don't bother with UpdateKeys
			this.getOptions();
			this.updateStatus();
			break;
			
			case "quit-application-granted":
			// Reset counts to zero and disable undo functionality
			this.prefs.setIntPref("selectedWords", 0);
			this.prefs.setIntPref("previousWords", -1);
			this.prefs.setIntPref("totalWords", 0);
			this.prefs.setIntPref("previousTotal", -1);
			this.prefs.setIntPref("selectedChars", 0);
			this.prefs.setIntPref("previousChars", -1);
			this.prefs.setIntPref("totalChars", 0);
			this.prefs.setIntPref("previousCharTotal", -1);
			this.prefs.setBoolPref("manualReset", true);
			this.prefs.setBoolPref("previousManualReset", true);
			this.observerService.removeObserver(wcp_quitObserver, "quit-application-granted");
			break;
			
			default:
			// do nothing
		}
	}

};

// SJW:20080709
// Ah, the joys of revisiting one's first extension. Switched over to using keyword "this"
// in the WordCountPlus var, and moved these event-triggered calls into functions so they
// get the right context (otherwise "this" points to the window, not the WCP object).
window.addEventListener("load", function() { WordCountPlus.init();}, false);
window.addEventListener("focus", function() { WordCountPlus.showPanel();}, false);	